﻿using ModuleTools.BindingModels;
using ModuleTools.Enums;
using ModuleTools.Interfaces;
using ModuleTools.Models;
using ModuleTools.ViewModels;
using System;

namespace ModuleTools.BusinessLogics
{
	/// <summary>
	/// Базовый класс для логики сущности
	/// </summary>
	/// <typeparam name="G"></typeparam>
	/// <typeparam name="S"></typeparam>
	/// <typeparam name="L"></typeparam>
	/// <typeparam name="E"></typeparam>
	public class GenericBusinessLogic<G, S, L, E> : CoreBusinessLogic
		where G : GetBindingModel
		where S : SetBindingModel
		where L : ListViewModel<E>
		where E : ElementViewModel
	{
		/// <summary>
		/// Сервис с хранилищем данных
		/// </summary>
		protected IGenerticEntityService<G, S> Service { get; set; }

		/// <summary>
		/// Тип операции, скоторым работает логика
		/// </summary>
		protected readonly AccessOperation _serviceOperation;

		/// <summary>
		/// Название сущности
		/// </summary>
		protected readonly string _entity;

		/// <summary>
		/// Возможен ли просмотр без авторизации
		/// </summary>
		protected bool _allowSimpleView = true;

		public GenericBusinessLogic(IGenerticEntityService<G, S> service, string entity, AccessOperation serviceOperation)
		{
			Service = service;
			_entity = entity;
			_serviceOperation = serviceOperation;
		}

		/// <summary>
		/// Проверка доступности операции для пользователя
		/// </summary>
		/// <param name="model"></param>
		/// <param name="type"></param>
		/// <returns></returns>
		protected bool NoAccess(AccessBindingModel model, AccessType type)
		{
			if (_security.CheckAccess(new SecurityManagerCheckAccessModel(model, _serviceOperation, type, _entity)))
			{
				return false;
			}
			Errors.Add(("Ошибка безопасности", _security.ErrorMessage));
			return true;
		}

		/// <summary>
		/// Получение списка записей
		/// </summary>
		/// <param name="model"></param>
		/// <returns></returns>
		public L GetList(G model)
		{
			Errors.Clear();
			try
			{
				model.HaveRight = !NoAccess(model, AccessType.View);
				if (model.HaveRight && !_allowSimpleView)
				{
					throw new MethodAccessException("Нет прав на получение списка");
				}
				var result = Service.Read(model);
				if (!result.IsSucceeded)
				{
					Errors.AddRange(Errors);
					return null;
				}

				return result.Result as L;
			}
			catch (Exception ex)
			{
				Errors.Add(("Ошибка получения", ex.Message));
			}
			return null;
		}

		/// <summary>
		/// Получение записи
		/// </summary>
		/// <param name="model"></param>
		/// <returns></returns>
		public E GetElement(G model)
		{
			Errors.Clear();
			try
			{
				model.HaveRight = !NoAccess(model, AccessType.View);
				if (model.HaveRight && !_allowSimpleView)
				{
					throw new MethodAccessException("Нет прав на получение списка");
				}
				var result = Service.Read(model);
				if (!result.IsSucceeded)
				{
					Errors.AddRange(Errors);
					return null;
				}
				return result.Result as E;
			}
			catch (Exception ex)
			{
				Errors.Add(("Ошибка получения", ex.Message));
			}
			return null;
		}

		/// <summary>
		/// Создание записи
		/// </summary>
		/// <param name="model"></param>
		/// <returns></returns>
		public E Create(S model)
		{
			Errors.Clear();
			try
			{
				if (NoAccess(model, AccessType.Change))
				{
					return null;
				}
				var result = Service.Create(model);
				if (!result.IsSucceeded)
				{
					Errors.AddRange(Errors);
					return null;
				}

				return result.Result as E;
			}
			catch (Exception ex)
			{
				Errors.Add(("Ошибка создания", ex.Message));
			}
			return null;
		}

		/// <summary>
		/// Изменение записи
		/// </summary>
		/// <param name="model"></param>
		/// <returns></returns>
		public E Update(S model)
		{
			Errors.Clear();
			try
			{
				if (NoAccess(model, AccessType.Change))
				{
					return null;
				}
				var result = Service.Update(model);
				if (!result.IsSucceeded)
				{
					Errors.AddRange(Errors);
					return null;
				}

				return result.Result as E;
			}
			catch (Exception ex)
			{
				Errors.Add(("Ошибка изменения", ex.Message));
			}
			return null;
		}

		/// <summary>
		/// Удаление записи
		/// </summary>
		/// <param name="model"></param>
		/// <returns></returns>
		public bool Delete(G model)
		{
			Errors.Clear();
			try
			{
				if (NoAccess(model, AccessType.Delete))
				{
					return false;
				}
				var result = Service.Delete(model);
				if (!result.IsSucceeded)
				{
					Errors.AddRange(Errors);
					return false;
				}

				return true;
			}
			catch (Exception ex)
			{
				Errors.Add(("Ошибка удаления", ex.Message));
			}
			return false;
		}
	}
}