﻿using ModuleTools.BusinessLogics;
using ModuleTools.Enums;
using ModuleTools.Extensions;
using ModuleTools.Models;
using SecurityBusinessLogic.BindingModels;
using SecurityBusinessLogic.Interfaces;
using System;
using System.IO;

namespace SecurityBusinessLogic.BusinessLogics
{
	/// <summary>
	/// Логика работы с бекапом
	/// </summary>
	public class BackupBusinessLogic : CoreBusinessLogic
	{
		/// <summary>
		/// Серивс для работы с бекапом
		/// </summary>
		private readonly IBackupService _service;

		/// <summary>
		/// Логика работы с бекапом
		/// </summary>
		/// <param name="service"></param>
		public BackupBusinessLogic(IBackupService service) => _service = service;

		/// <summary>
		/// Создание бекапа с данными
		/// </summary>
		/// <param name="model"></param>
		/// <returns></returns>
		public bool CreateBackUp(BackupBindingModel model)
		{
			if (model.FolderName.IsEmpty())
			{
				Errors.Add(("Ошибка", "Путь до папки не указан"));
				return false;
			}
			try
			{
				if (NoAccess())
				{
					return false;
				}
				var result = _service.CreateBackUp(model);
				if (!result.IsSucceeded)
				{
					Errors.AddRange(result.Errors);
					return false;
				}
				return true;
			}
			catch (Exception ex)
			{
				Errors.Add(("Ошибка", ex.Message));
				return false;
			}
		}

		/// <summary>
		/// Восстанволение данных через бекап
		/// </summary>
		/// <param name="model"></param>
		/// <returns></returns>
		public bool RestoreBackUp(BackupBindingModel model)
		{
			if (model.FolderName.IsEmpty() && model.ArchiveFileName.IsEmpty())
			{
				Errors.Add(("Ошибка", "Путь до папки/архива не указан"));
				return false;
			}
			if (model.ArchiveFileName.IsNotEmpty() && !File.Exists(model.ArchiveFileName))
			{
				Errors.Add(("Ошибка", "Файл с архивом не найден"));
				return false;
			}
			if (model.ArchiveFileName.IsEmpty() && !Directory.Exists(model.FolderName))
			{
				Errors.Add(("Ошибка", "Папка с даными не найдена"));
				return false;
			}
			try
			{
				if (NoAccess())
				{
					return false;
				}
				var result = _service.RestoreBackUp(model);
				if (!result.IsSucceeded)
				{
					Errors.AddRange(result.Errors);
					return false;
				}
				return true;
			}
			catch (Exception ex)
			{
				Errors.Add(("Ошибка", ex.Message));
				return false;
			}
		}

		/// <summary>
		/// Проверка доступности операции для пользователя
		/// </summary>
		/// <returns></returns>
		private bool NoAccess()
		{
			if (_security.CheckAccess(new SecurityManagerCheckAccessModel(null, AccessOperation.РаботасБекапом, AccessType.Delete, "бекап")))
			{
				return false;
			}
			Errors.Add(("Ошибка безопасности", _security.ErrorMessage));
			return true;
		}
	}
}